@@ -8,34 +8,20 @@ __author__ = 'Sam Forester'
 __email__ = 'sam.forester@utah.edu'
 __copyright__ = 'Copyright (c) 2020 University of Utah, Marriott Library'
 __license__ = 'MIT'
-__version__ = "1.2.3"
+__version__ = "1.2.4"
 
-import copy
 import getpass
 import logging
 import plistlib
-import itertools
-from os import path
+import keyring
+from os import path, remove
 LINUX_PREFS_TILDA = '~/.edu.utah.mlib.jamfutil.plist'
 MACOS_PREFS_TILDA = '~/Library/Preferences/edu.utah.mlib.jamfutil.plist'
 AUTOPKG_PREFS_TILDA = '~/Library/Preferences/com.github.autopkg.plist'
 JAMF_PREFS = '/Library/Preferences/com.jamfsoftware.jamf.plist'
-MAGIC = (125, 137, 82, 35, 210, 188, 221, 234, 163, 185, 31)
 logging.getLogger(__name__).addHandler(logging.NullHandler())
 
 
-class Error(Exception):
-    pass
-
-
-class CorruptedConfigError(Error):
-    pass
-
-
-class ValidationError(Error):
-    pass
-
-
 class Config:
     def __init__(self,
                  config_path=None,
@@ -49,27 +35,26 @@ class Config:
         self.hostname = hostname
         self.username = username
         self.password = password
-        self._credentials = None
         if not self.hostname and not self.username and not self.password:
-            macos_prefs = path.expanduser(MACOS_PREFS_TILDA)
             if not config_path:
+                macos_prefs = path.expanduser(MACOS_PREFS_TILDA)
                 linux_prefs = path.expanduser(LINUX_PREFS_TILDA)
                 autopkg_prefs = path.expanduser(AUTOPKG_PREFS_TILDA)
                 if path.exists(macos_prefs):
                     if explain:
-                        print("Using "+macos_prefs)
+                        print("Using macos: "+macos_prefs)
                     config_path = macos_prefs
                 elif path.exists(linux_prefs):
                     if explain:
-                        print("Using "+linux_prefs)
+                        print("Using linux: "+linux_prefs)
                     config_path = linux_prefs
                 elif path.exists(autopkg_prefs):
                     if explain:
-                        print("Using "+autopkg_prefs)
+                        print("Using autopkg: "+autopkg_prefs)
                     config_path = autopkg_prefs
                 elif path.exists(JAMF_PREFS):
                     if explain:
-                        print("Using "+JAMF_PREFS)
+                        print("Using jamf: "+JAMF_PREFS)
                     config_path = JAMF_PREFS
                 else:
                     if explain:
@@ -89,10 +74,23 @@ class Config:
                     prefs = plistlib.load(fptr)
                     fptr.close()
                     if 'JSSHostname' in prefs:
+                        if 'Credentials' in prefs:
+                            cmessage = f"""
+ATTENTION
+To improve security with storing credentials used with the jctl tool, we have
+deprecated the use of a property list file for storing configuration
+information and have migrated to use the Python keyring library provides an
+easy way to access the system keyring service from python. It can be used with
+the macOS Keychain and Linux KWallet.
+
+Please delete the the configuration at {config_path} and recreate it using
+the "./jamf/setconfig.py" script.
+"""
+                            raise Exception(cmessage)
                         self.hostname = prefs['JSSHostname']
-                        c = prefs.get('Credentials', {})
-                        self._credentials = Credentials(c, callback=transposition(MAGIC))
-                        (self.username, self.password) = self.read_credentials(self.hostname)
+                        self.username = prefs['Username']
+                        self.password = keyring.get_password(self.hostname,
+                                                             self.username)
                     elif 'JSS_URL' in prefs:
                         self.hostname = prefs["JSS_URL"]
                         self.username = prefs["API_USERNAME"]
@@ -103,6 +101,8 @@ class Config:
                 else:
                     self.log.debug(f"file not found: {config_path}")
 
+        self.config_path = config_path
+
         # Prompt for any missing prefs
         if self.prompt:
             if not self.hostname:
@@ -112,80 +112,22 @@ class Config:
             if not self.password:
                 self.password = getpass.getpass()
         elif not self.hostname and not self.username and not self.password:
-#             raise FileNotFoundError(config_path)
-            print("No jamf config file could be found and prompt is off.")
-            exit(1)
+            raise Exception('No jamf config file could be found and prompt is off.')
 
     def save(self, config_path=None):
-        if not self._credentials:
-            self._credentials = Credentials({}, callback=transposition(MAGIC))
-        self._credentials.register(self.hostname, (self.username, self.password))
-        creds = bytes(self._credentials)
+        keyring.set_password(self.hostname, self.username, self.password)
         data = {
             'JSSHostname': self.hostname,
-            'Credentials': creds
+            'Username': self.username
         }
         self.log.info(f"saving: {config_path}")
         fptr = open(config_path, 'wb')
         plistlib.dump(data, fptr)
         fptr.close()
 
-    def read_credentials(self, hostname):
-        return tuple(self._credentials.retrieve(hostname))
-
-    def reset(self):
-        self.save()
-        self._credentials = Credentials({}, callback=transposition(MAGIC))
-
-
-class Credentials:
-    def __init__(self, data, callback=None):
-        self.data = data
-        if isinstance(data, (list, tuple, dict)):
-            self.data = copy.deepcopy(data)
-        elif isinstance(data, bytes):
-            self.data = plistlib.loads(data)
-        self.transpose = callback if callback else lambda x: x
-
-    def retrieve(self, key):
-        value = self.data[key]
-        data = self.transpose(value)
-        try:
-            return plistlib.loads(data)
-        except plistlib.InvalidFileException:
-            try:
-                return data.decode('utf-8')
-            except AttributeError:
-                return data
-
-    def register(self, key, value):
-        if isinstance(value, Credentials):
-            value = bytes(value)
-        else:
-            value = bytes(Credentials(value, self.transpose))
-        self.data[key] = self.transpose(value)
-
-    def reset(self):
-        self.data = {}
-
-    def __bytes__(self):
-        return plistlib.dumps(self.data, fmt=plistlib.FMT_BINARY)
-
-    def __bool__(self):
-        return True if self.data else False
-
-
-def transposition(key):
-    if isinstance(key, str):
-        key = bytes(key, encoding='utf-8')
-    if not all(isinstance(x, int) for x in key):
-        raise ValueError(f"invalid key: {key!r}")
-
-    def _wrapped(data):
-        if isinstance(data, str):
-            data = bytes(data, encoding='utf-8')
-        return bytes(x ^ y for x, y in zip(data, itertools.cycle(key)))
-    return _wrapped
+    def reset(self, path):
+        keyring.delete_password(self.hostname, self.username)
+        remove(path)
 
 
 def prompt_hostname():
